import {
  defineComponent,
  onBeforeUpdate,
  onUnmounted,
  watch,
} from "vue-demi";

let idCount = 0 ;

export default defineComponent({
  props: {
    name: {
      type: String,
      default: () => null,
    },
    height: {
      type: String,
      default: () => "auto",
    },
    scroll: {
      type: Boolean,
      default: () => false,
    },
    items: {
      type: Array ,
      default: () => undefined,
    },
    url: {
      type: String,
      default: () => null,
    },
  },
  setup(props, { slots, expose }) {
    //     const { proxy } = getCurrentInstance();
    //
    //     const id = modService.addInvoker(proxy);
    const id = ++idCount;
    const isLocalInvoker = false;
    let modContext: any = null;
    const invokeMethods: { method: string; args: any[] }[] = [];
    let isTemplate = false;

    window["MES_MOD_INVOKERS"] ||= {};
    window["MES_MOD_INVOKERS"][id] = {
      getRenderContext,
      initFinish,
      modContext,
    };

    function updateComponent() {
      if (modContext) {
        // 判断子页面vue对象是否存在
        modContext.renderVersion += 1;
        modContext.$forceUpdate(); // 强制渲染
        // 重新计算el-button的computed
        const list = [modContext];

        while (list.length != 0) {
          const item = list.pop();
          list.push(...item.$children);
          const tag = item.$options._componentTag;

          if (tag == "el-button" || tag == "ElButton") {
            if (item.disabled) item._computedWatchers.buttonDisabled.run();
            console.log("[Invoker]: force recalculate computed property...");
          }
        }
      }
    }

    // 子组件调用
    function eventCallback(type: string, ...args: any[]) {
      switch (type) {
        case "created":
          initFinish(args[0]);
          break;
      }
    }
    function initFinish(context) {
      modContext = context;
      let m: { method: string; args: any[] } | undefined;
      while ((m = invokeMethods.pop())) {
        invokeMethod(m.method, m.args);
      }
      console.log("invoker init finish");
    }
    function invokeMethod(
      method: string,
      args: any[] = [],
      callback: ((result: any) => void) | null = null
    ) {
      // if(!this.component) throw new Error('子组件不支持此方法调用');
      if (modContext) {
        console.log("method", modContext.$children);
        const result = modContext.$children[0][method](...args);
        if (callback) {
          callback(result);
        }
      } else {
        invokeMethods.push({ method, args });
      }
    }
    function getRefs() {
      return modContext.$refs;
    }

    // vue3 渲染上下文
    function getRenderContext() {
      return props.items;
    }

    expose({
      getRefs,
      getRenderContext,
      eventCallback,
      initFinish,
    });

    watch(
      () => props.items,
      () => {
        updateComponent();
      }
    );

    onBeforeUpdate(() => {
      console.log("invoker-beforeUpdate", slots.default);
      if (isTemplate) {
        // 只更新slots变更
        updateComponent();
      }
    });
    onUnmounted(() => {
      console.log("invoker-destroyed");
      modContext && modContext.$destroy();
    });

    return (h) => {
      if (isLocalInvoker && slots.default) {
        return <div>{slots.default()}</div>;
      }

      return (
        <iframe
          src={props.url.replace('{id}',id.toString())}
          scrolling={props.scroll ? "yes" : "no"}
          style={`border: none; width: 100%; height: ${props.height}; `}
        ></iframe>
      );
    };
  },
});
